@@ -1,13 +1,14 @@
+from typing import *
+
+import pytorch_lightning as pl
 import torch
 from torch import nn
 from torch.nn import functional as F
-import pytorch_lightning as pl
 
-from typing import *
+from df2d.util import heatmap2points, tensorboard_plot_image
 
 device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
 
-
 class Drosophila2DPose(pl.LightningModule):
     def __init__(
         self,
@@ -43,31 +44,82 @@ class Drosophila2DPose(pl.LightningModule):
     def forward(self, x):
         return self.model(x.float().to(device))[-1]
 
+    def mse(self, hm_hat, y):
+        y_hat, _ = heatmap2points(hm_hat).cuda()
+        mse = torch.norm(y_hat - y, dim=2)
+        return mse.mean()
+
     def configure_optimizers(self):
-        optimizer = torch.optim.RMSprop(self.parameters(), lr=1e-4, weight_decay=1e-5,)
+        optimizer = torch.optim.RMSprop(
+            self.parameters(),
+            lr=1e-4,
+            weight_decay=1e-5,
+        )
         scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(
             optimizer, verbose=True, patience=5
         )
-        return optimizer, scheduler
+        return {
+            "optimizer": optimizer,
+            "lr_scheduler": scheduler,
+            "monitor": "train/loss",
+        }
+
+    def loss(self, hm_hat, hm):
+        # fmt: off
+        # missing joint predictions are set to all zeros, which is the same of the gt heatmap
+        #   therefore giving zero loss
+        missing_joints = torch.all(torch.flatten(hm, 2) == 0, dim=2, keepdim=True)
+        missing_joints = missing_joints.float()
+        missing_joints = missing_joints.unsqueeze(-1)
+        hm_hat = hm_hat * (1 - missing_joints)
+        # fmt: on
+        return F.mse_loss(hm, hm_hat)
 
     def training_step(self, train_batch, batch_idx):
-        x, y = train_batch
-        x_hat = self.model(x)
-        loss = F.mse_loss(x_hat, x)
-        self.log("train_loss", loss)
+        x, d, y, hm = train_batch
+        x = x.to(device).float()
+        hm = hm.to(device).float()
+        hm_hat = self.model(x)
+
+        loss = self.loss(hm_hat[0], hm)
+        for idx in range(1, len(hm_hat)):
+            loss += self.loss(hm_hat[idx], hm)
+        self.log("train/loss", loss, on_step=True)
+
+        # log images
+        if batch_idx == 0:
+            tensorboard_plot_image(
+                self.logger.experiment,
+                torch.cat([x[:, [0], ::4, ::4], hm.sum(dim=1, keepdims=True)], dim=2),
+                name="data",
+                epoch=self.current_epoch,
+                scale_each=True,
+            )
+
+            # log mse loss
+            self.log(name="train/mse", value=self.mse(hm_hat[-1], d), prog_bar=True)
+
         return loss
 
     def validation_step(self, val_batch, batch_idx):
-        x, y = val_batch
-        x_hat = self.model(x)
-        loss = F.mse_loss(x_hat, x)
-        self.log("val_loss", loss)
+        x, d, y, hm = val_batch
+        x = x.to(device).float()
+        hm = hm.to(device).float()
+        hm_hat = self.model(x.float())
+
+        loss = self.loss(hm_hat[0], hm)
+        for idx in range(1, len(hm_hat)):
+            loss += self.loss(hm_hat[idx], hm)
+        self.log("val/loss", loss, on_step=True)
+
+        # log mse loss
+        self.log(name="val/mse", value=self.mse(hm_hat[-1], d), prog_bar=True)
+
         return loss
 
 
 """
-Hourglass network inserted in the pre-activated Resnet 
-(c) YANG, Wei 
+Hourglass network inserted in the pre-activated Resnet
 """
 import torch.nn as nn
 import torch.nn.functional as F
@@ -227,7 +279,11 @@ class HourglassNet(nn.Module):
     def _make_fc(self, inplanes, outplanes):
         bn = nn.BatchNorm2d(inplanes)
         conv = nn.Conv2d(inplanes, outplanes, kernel_size=1, bias=True)
-        return nn.Sequential(conv, bn, self.relu,)
+        return nn.Sequential(
+            conv,
+            bn,
+            self.relu,
+        )
 
     def forward(self, x):
         out = []
